<?php
/*
	$Project: CSV Uniexport $
	$Author: karapuz team <support@ka-station.com> $
	$Version: 2.0.0.6 $ ($Revision: 158 $)

*/

namespace extension\ka_extensions\ka_uniexport;

abstract class ControllerUEInstaller extends \KaInstaller {

	protected $min_store_version = '3.0.0.0';
	protected $max_store_version = '3.0.3.99';
	protected $min_ka_extensions_version = '4.1.1.11';
	protected $max_ka_extensions_version = '4.1.1.99';
	protected $module_prefix = '';
	
	protected $tables;

	protected function onLoad() {

		$this->load->model('setting/setting');
		
		$this->load->language('extension/ka_extensions/ka_uniexport/settings');
			
 		$this->tables = array(
 		
 			'product' => array(
 				'fields' => array(),
 				'indexes' => array(
 					'model' => array(
	 					'query' => "ALTER TABLE " . DB_PREFIX . "product ADD INDEX (`model`)",
	 				)
 				)
 			),
 			
 			'ka_uniexport_profiles' => array(
 				'is_new' => true,
 				'fields' => array(
  					'export_profile_id' => array(
  						'type' => 'int(11)',
  					),
  					'name' => array(
  						'type' => 'varchar(128)',
  					),
  					'entity_code' => array(
  						'type' => 'varchar(128)',
  					),
  					'params' => array(
  						'type' => 'mediumtext',
  					),
  				),
				'indexes' => array(
					'PRIMARY' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_uniexport_profiles` ADD PRIMARY KEY (`export_profile_id`)",
					),
					'name' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_uniexport_profiles` ADD INDEX (`name`)",
					),
					'entity_code' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_uniexport_profiles` ADD INDEX (`entity_code`)",
					),
				),
			),
		);

		$this->tables['ka_uniexport_profiles']['query'] = "
			CREATE TABLE `" . DB_PREFIX . "ka_uniexport_profiles` (
			  `export_profile_id` int(11) NOT NULL auto_increment,
			  `name` varchar(128) NOT NULL,
			  `entity_code` varchar(128) NOT NULL,
			  `params` mediumtext NOT NULL,
			  PRIMARY KEY  (`export_profile_id`),
			  KEY `name` (`name`)
			) DEFAULT CHARSET=utf8 COLLATE=utf8_bin
		";
		
		return true;
	}

	
	protected function processPost() {
		$val = max(5, $this->request->post['ka_ue_update_interval']);
		$this->request->post['ka_ue_update_interval'] = min(25, $val);
		
		if (empty($this->request->post['ka_ue_prefix_with_space'])) {
			$this->request->post['ka_ue_prefix_with_space'] = '';
		}

		if (empty($this->request->post['ka_ue_data_in_one_cell'])) {
			$this->request->post['ka_ue_data_in_one_cell'] = '';
		}
		
		$this->editSetting($this->request->post);
	}
	
	protected function prepareData() {
		$heading_title = $this->getTitle();
		$this->document->setTitle($heading_title);
	
		$this->data['heading_title']         = $heading_title;
		$this->data['extension_version']     = $this->extension_version;
	
		$this->data['button_save']           = $this->language->get('button_save');
		$this->data['button_cancel']         = $this->language->get('button_cancel');

		$this->data['ka_ue_update_interval']   = $this->getConfig('ka_ue_update_interval');
		$this->data['ka_ue_prefix_with_space'] = $this->getConfig('ka_ue_prefix_with_space');
		$this->data['ka_ue_data_in_one_cell']  = $this->getConfig('ka_ue_data_in_one_cell');
		$this->data['ka_ue_general_sep']       = $this->getConfig('ka_ue_general_sep');

 		$this->data['breadcrumbs'] = array();
 		$this->data['breadcrumbs'][] = array(
	   		'text'      => $this->language->get('text_home'),
			'href'      => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true),
 		);

	  	$this->data['breadcrumbs'][] = array(
   			'text'      => $this->language->get('Ka Extensions'),
			'href'      => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'], true),
  		);
  		
 		$this->data['breadcrumbs'][] = array(
   			'text'      => $heading_title,
 		);
		
		$this->children = array(
			'common/header',
			'common/column_left',
			'common/footer'
		);

		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_uniexport', 'user_token=' . $this->session->data['user_token'], true);
		$this->data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=ka_extensions', true);
				
		$this->template = 'extension/ka_extensions/ka_uniexport/settings';
	}	
	
	
	public function index() {

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {

			$this->processPost();
		
			$this->addTopMessage($this->language->get('Settings have been stored sucessfully.'));
			$this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token']  . '&type=ka_extensions', true));
		}

		$this->prepareData();
		
		$this->setOutput();
	}

	abstract protected function validate();
	
	public function install() {

		if (!parent::install()) {
			return false;
		}

		$settings = array(
			'ka_ue_update_interval'   => 10,
			'ka_ue_prefix_with_space' => 'N',
			'ka_ue_general_sep'       => ':::',
			'ka_ue_data_in_one_cell'    => 'Y',
		);
		$this->editSetting($settings);

		return true;
	}

	
	protected function getConfig($setting) {
		return $this->config->get($this->module_prefix . '_' . $setting);
	}
	
	protected function editSetting($settings) {
		
		if (!empty($settings)) {
			$new_settings = array();
			foreach ($settings as $k => $v) {
				$new_settings[$this->module_prefix . '_' . $k] = $v;
			}
			$settings = $new_settings;
		}
	
		return $this->model_setting_setting->editSetting($this->module_prefix, $settings);
	}
}