<?php
/*
	$Project: CSV Uniexport $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 2.0.0.6 $ ($Revision: 158 $)
*/

namespace extension\ka_extensions\ka_uniexport;

class ControllerUniexport extends \KaController { 

	protected $tmp_dir;
	protected $store_root_dir;
	protected $store_images_dir;
	
	protected $export_model_name = '';
	protected $export_model      = null;
	
	protected $main_entity;
	
	// session variables
	//
	protected $params;

	protected function onLoad() {
	
		$this->params = &$this->getSession('params');
	
		$this->kamodel('export');
		
		$this->load->language('extension/ka_extensions/ka_uniexport/ka_uniexport');
		$this->data['heading_title'] = $this->language->get('heading_title');

		$this->kaformat         = new \KaFormat($this->config->get('config_language_id'));
		$this->tmp_dir          = DIR_CACHE;
		$this->store_root_dir   = dirname(DIR_APPLICATION);
		$this->store_images_dir = dirname(DIR_IMAGE) . DIRECTORY_SEPARATOR . basename(DIR_IMAGE);
	}


	protected function getStores() {

		$this->load->model('setting/store');
		$stores = $this->model_setting_store->getStores();

		$stores[] = array(
			'store_id' => 0,
			'name'     => $this->config->get('config_name') . $this->language->get('text_default'),
			'url'      => HTTP_CATALOG,
		);

		$stores[] = array(
			'store_id' => -1,
			'name'     => '--- unassigned --',
			'url'      => HTTP_CATALOG,
		);
		
		return $stores;
	}

	
	protected function initStep1Data() {
		if (empty($this->params) || ($this->request->server['REQUEST_METHOD'] == 'GET' && empty($this->session->data['save_params']))) {
			$this->params = array(
				'main_entity_to_each_row' => false,
				'main_entity_code'    => '',
				'charset'             => 'UTF-8', // 'UTF-16LE' - MS Excel
				'file_settings'       => 'custom',
				'charset_option'      => 'predefined',
				'delimiter'           => 's',
				'store_id'            => 0,
				'image_path'          => 'path',
				'profile_name'        => '',
				'profile_id'          => '',
				'language_id' => 0,
				'copy_path'           => '',
			);
		}
		$this->session->data['save_params'] = false;		
	
		return true;
	}
	

	protected function processStep1Post() {

		$msg = '';
	
		if ($this->request->post['mode'] == 'load_profile') {
			$this->params = $this->kamodel_export->getProfileParams($this->request->post['profile_id']);
			$this->params['profile_id'] = $this->request->post['profile_id'];
			$this->session->data['save_params'] = true;

			$this->addTopMessage("Profile has been loaded succesfully");
			return $this->response->redirect($this->getPageURL());

		} elseif ($this->request->post['mode'] == 'delete_profile') {
			$this->kamodel_export->deleteProfile($this->request->post['profile_id']);
			$this->session->data['save_params'] = true;
			$this->addTopMessage("Profile has been deleted succesfully");
			
			return $this->response->redirect($this->getPageURL());
		}

		$this->params['file'] = $this->kamodel_export->tempname(DIR_CACHE, "uniexport_" . date("Y-m-d") . '_', 'csv');
			
		$this->params['delimiter']      = $this->request->post['delimiter'];
		$this->params['language_id']    = $this->request->post['language_id'];
		$this->params['copy_path']      = $this->request->post['copy_path']; 
		
		$this->params['store_id']       = $this->request->post['store_id'];
		$this->params['image_path']     = $this->request->post['image_path'];
		
		if (isset($this->request->post['main_entity_to_each_row'])) {
			$this->params['main_entity_to_each_row'] = true;
		} else {
			$this->params['main_entity_to_each_row'] = false;
		}
		
		$this->params['charset_option'] = $this->request->post['charset_option']; 
		if ($this->params['charset_option'] == 'predefined') {
			$this->params['charset'] = $this->request->post['charset']; 
		} else {
			$this->params['charset'] = $this->request->post['custom_charset']; 
		}

		if (empty($this->params['file'])) {
			$msg = "Cannot create a temporary file. Check 'cache' directory permissions.";
		}

		if (!empty($this->request->post['entity_params'])) {			
			$this->params['entity_params'] = $this->request->post['entity_params'];
		}
		
		return $msg;
	}

	
	protected function prepareStep1Data() {
		
		$profiles = $this->kamodel_export->getProfiles($this->params['main_entity_code']);
		$this->data['profiles'] = $profiles;

		$this->params['step'] = 1;

		$this->load->model('localisation/language');
		$this->data['languages'] = $this->model_localisation_language->getLanguages();

		$this->load->model('setting/store');
		$this->data['stores']    = $this->getStores();
		
		$this->data['heading_title'] = $this->language->get('heading_title');
		$this->document->setTitle($this->data['heading_title']);
		
		$this->data['store_images_dir'] = $this->store_images_dir;
		$this->data['store_root_dir']   = $this->store_root_dir;

		$this->data['charsets'] = $this->kamodel_export->getCharsets();
		
		if (!isset($this->params['entity_params'])) {
			$this->params['entity_params'] = array();
		}
		
		$this->data['params'] = $this->params;
						
		$this->data['breadcrumbs'] = array();
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('text_home'),
			'href'      => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true),
		);
		
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->data['heading_title'],
			'href'      => $this->getPageURL(),
		);
		
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('Step 1 of 3'),
		);

		$this->main_entity = $this->kamodel_export->loadEntityModel($this->params['main_entity_code']);
		
		$this->data['entity_param_fields'] = $this->main_entity->getParamFields();

		$this->children = array(
			'common/header',
			'common/footer',
			'common/column_left',
		);
		
		$this->data['action']    = $this->getPageURL();
		$this->data['step_data'] = $this->load->view('extension/ka_extensions/ka_uniexport/step1', $this->data);
		
		$this->template = 'extension/ka_extensions/ka_uniexport/export';
	}


	
	// it should be overwritten in the child class
	//	
	protected function getPageURL($page = '') {
		$url = 'extension/ka_extensions/ka_uniexport/ka_uniexport';
		if (!empty($page)) {
			$url .= '/' . $page;
		}
		
		$extra_params = '';
		if ($page == 'stat') {
			$extra_params = '&format=raw&tmpl=component';
		}
		
		return $this->url->link($url, 'user_token=' . $this->session->data['user_token'] . $extra_params, true);
	}
	
	
	public function index() { // step 1

		$this->initStep1Data();
	
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {

			$msg = $this->processStep1Post();

			if (!empty($msg)) {
				$this->addTopMessage($msg, 'E');
				$this->session->data['save_params'] = true;
			 	return $this->response->redirect($this->getPageURL());
			}

			return $this->response->redirect($this->getPageURL('step2'));
		}

		$this->prepareStep1Data();
		
		$this->response->setOutput($this->render());
	}

	
	public function initStep2Data() {
		if (!$this->kamodel_export->setMainEntity($this->params['main_entity_code'])) {
			die(__METHOD__ . ': main entity not set');
		}

		$this->params['step'] = 2;
		
		$this->main_entity = $this->kamodel_export->loadEntityModel($this->params['main_entity_code']);
		
		return true;
	}

	public function processStep2Post() {

		$sets = $this->kamodel_export->getFieldSets();
	
		$this->params['columns'] = $this->request->post['columns'];
		
		$columns_found = false;
		$key_found = false;

		if (!empty($this->params['columns'])) {

			// 1) fill in the type column
			// 2) trim custom column names
			//
			foreach ($sets as $s) {
			
				$set_code = $s['code'];
				if (!empty($this->params['columns'][$set_code])) {
					foreach ($this->params['columns'][$set_code] as $col_name => &$col_data) {
					
						if (!empty($s['fields'][$col_name]['type'])) {
							$col_data['type'] = $s['fields'][$col_name]['type'];
						}
						
						if (!empty($s['fields'][$col_name]['is_key'])) {
							$col_data['is_key'] = true;
							if (!empty($col_data['checked']) && $this->params['main_entity_code'] == $set_code) {
								$key_found = true;
							}
						}
						if (isset($col_data['column'])) {
							$col_data['column'] = trim($col_data['column']);
						}
						if (!empty($col_data['checked'])) {
							$columns_found = true;
						}
					}
				}
			}
		}
		
		if ($this->request->post['mode'] == 'save_profile') {
		
			if (empty($this->request->post['profile_name'])) {
				$this->addTopMessage("Profile name is empty", "E");
				
			} else {
				if ($this->kamodel_export->setProfileParams($this->request->post['profile_id'], $this->request->post['profile_name'], $this->params)) {
					$this->addTopMessage("Profile has been saved succesfully");
				}
			}
			
			return $this->response->redirect($this->getPageURL('step2'));
		}

		$error_message = '';
		if (empty($columns_found)) {
			$error_message = "You have to map at least one column.";
		} elseif (empty($key_found)) {
			$error_message = "At least one required field has to be selected for export. The file cannot be generated.";
		}
					
		if ($error_message) {
			$this->addTopMessage($error_message, "E");
			return $this->response->redirect($this->getPageURL('step2'));
		}
	}


	public function prepareStep2Data() {
	
		$this->data['heading_title'] = $this->language->get('CSV ' . $this->main_entity->getEntityName() . ' Export');
		$this->document->setTitle($this->data['heading_title']);

		$sets = $this->kamodel_export->getFieldSets();
		
		$this->data['sets']   = $sets;
		$this->data['params'] = $this->params;
		
		if (!empty($this->params['columns'])) {
			$this->data['columns'] = $this->params['columns'];
		}
		
    	$this->data['action']      = $this->getPageURL('step2');
		$this->data['back_action'] = $this->getPageURL();

		$this->data['breadcrumbs'] = array();
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('text_home'),
			'href'      => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true),
		);
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->data['heading_title'],
			'href'      => $this->getPageURL(),
		);
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('Step 2 of 3'),
		);
		
		$this->data['step_data'] = $this->load->view('extension/ka_extensions/ka_uniexport/step2', $this->data);
		$this->children = array(
			'common/header',
			'common/footer',
			'common/column_left',
		);
		$this->template = 'extension/ka_extensions/ka_uniexport/export';
	}
	
	
	public function step2() { // step 2

		$this->initStep2Data();
	
		if (($this->request->server['REQUEST_METHOD'] == 'POST')) {
		
			$msg = $this->processStep2Post();

			if (!empty($msg)) {
				$this->addTopMessage($msg, 'E');
				$this->session->data['save_params'] = true;
			}
		
			return $this->response->redirect($this->getPageURL('step3'));
		}
		
		$this->prepareStep2Data();

		$this->response->setOutput($this->render());
	}

	public function prepareStep3Data() {

		$this->params['step'] = 3;

		$params = $this->params;

		$params['delimiter'] = strtr($params['delimiter'], array('c'=>',', 's'=>';', 't'=>"\t"));

		if (!$this->kamodel_export->initExport($params)) {
			die(__METHOD__ .': init fail');
			$this->addTopMessage($this->kamodel_export->getLastError(), 'E');
			return $this->response->redirect($this->getPageURL('step3'));
		}

		$this->main_entity = $this->kamodel_export->loadEntityModel($this->params['main_entity_code']);
		$this->data['heading_title'] = $this->language->get('CSV ' . $this->main_entity->getEntityName() . ' Export');
		$this->document->setTitle($this->data['heading_title']);

		$this->data['breadcrumbs'] = array();
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('text_home'),
			'href'      => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true),
		);
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->data['heading_title'],
			'href'      => $this->getPageURL(),
		);
		$this->data['breadcrumbs'][] = array(
			'text'      => $this->language->get('Step 3 of 3'),
		);	
				
		$this->data['done_action']   = $this->getPageURL();
    	$this->data['download_link'] = $this->getPageURL('download');

		$this->data['params'] = $this->params;

		$sec = $this->kamodel_export->getSecPerCycle();

		$this->data['update_interval']    = $sec.' - ' .($sec + 5);
 		$this->data['page_url'] = str_replace('&amp;', '&', $this->getPageURL('stat'));
 		
		$this->data['step_data'] = $this->load->view('extension/ka_extensions/ka_uniexport/step3', $this->data);
		$this->children = array(
			'common/header',
			'common/footer',
			'common/column_left',
		);

		$this->template = 'extension/ka_extensions/ka_uniexport/export';
	}
	
	
	public function step3() {

		$this->prepareStep3Data();

		return $this->response->setOutput($this->render());
	}

	
	/*
		The function is called by ajax script and it outputs information in json format.

		json format:
			status - in progress, completed, error;
			...    - extra export parameters.
	*/
	public function stat() {

		if ($this->params['step'] != 3) {
			$this->addTopMessage('This script can be requested at step 3 only', 'E');
			return $this->response->redirect($this->url->link('tool/ka_uniexport/step3', 'user_token=' . $this->session->data['user_token'], true));
		}

		$this->kamodel_export->processExport();

		$stat                  = $this->kamodel_export->getExportStat();
		$stat['messages']      = $this->kamodel_export->getExportMessages();
		$stat['time_passed']   = $this->kaformat->formatPassedTime(time() - $stat['started_at']);
		$stat['file_size']     = $this->kaformat->convertToMegabyte($stat['file_size']);

		if ($stat['status'] == 'completed' || empty($stat['products_total'])) {
			$stat['completion_at'] = sprintf("%.2f%%", 100);
		} else if ($stat['status'] == 'in_progress') {
			$stat['completion_at'] = sprintf("%.2f%%", $stat['products_processed'] * 100 / $stat['products_total']);
		}
	
 		$this->response->setOutput(json_encode($stat));
	}


	public function download() {

		$stat = $this->kamodel_export->getExportStat();

		if ($stat['status'] == 'completed') {
			$file = $this->params['file'];
			$name = "export_" . date("Y-m-d") . ".csv";

			if (!headers_sent()) {
				if (file_exists($file)) {
					header('Content-Description: File Transfer');
					header('Content-Type: application/octet-stream');
					header("Content-Disposition: attachment; filename=\"$name\"");
					header('Content-Transfer-Encoding: binary');
					header('Expires: 0');
					header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
					header('Pragma: public');
					header('Content-Length: ' . filesize($file));
					
					readfile($file, 'rb');
					
					exit;
				} else {
					exit('Error: Could not find file ' . $file . '!');
				}
			} else {
				exit('Error: Headers were already sent out!');
			}
		} else {
			$this->response->redirect($this->getPageURL('step3'));
		}
	}
}