<?php
/*
	$Project: CSV Uniexport $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 2.0.0.6 $ ($Revision: 158 $)
*/

namespace extension\ka_extensions\ka_uniexport;

abstract class ModelEntity extends \KaIterator  {

	protected $entity_code = '';
	protected $entity_name = '';
	
	protected $is_main_entity = false;
	protected $subentity_for = array();	

	public $min_export_version = '1.0.0';
	public $max_export_version = '9.9.9';
	
	// we keep copies of entities for each subentity to prevent mixing conditions and 
	// other parameters
	//
	protected $subentities = array();
	
	protected $export_fields; //?
	protected $conditions;
	
	protected $export_model;

	
	protected function onLoad() {
		parent::onLoad();
		
		$this->http_image = HTTP_CATALOG . 'image/';
	}
	
	public function getEntityCode() {
		return $this->entity_code;
	}
	
	public function getEntityName() {
	
		if (empty($this->entity_name)) {
			$entity_name = $this->entity_code;
			$entity_name = ucwords(str_replace('_', ' ', $entity_name));
		} else {
			$entity_name = $this->entity_name;
		}
	
		return $entity_name;
	}

	function buildQuery() {
		$this->defineQuery();
	}
	
	/*
		this is a setParams() function actually.
	*/
	function setConditions($conditions) {		
		$this->conditions = $conditions;
		$this->buildQuery();
	}

	function setExportModel(&$model) {
		$this->export_model = &$model;
	}
	
	function setExportFields($fields) {
		$this->export_fields = $fields;
	}
	
	function getSubentities($checked_only = false) {

		if ($checked_only) {
			if (!empty($this->subentities['checked_only'])) {
				return $this->subentities['checked_only'];
			}
		} else {
			if (!empty($this->subentities['all'])) {
				return $this->subentities['all'];
			}
		}
	
		$entities = $this->export_model->getRegisteredEntities();

		$subentities = array();
		foreach ($entities as $ent) {
		
			if ($checked_only && empty($this->params['columns'][$ent->getEntityCode()])) {
				continue;
			}
		
			if (!$ent->isSubentityFor($this->entity_code)) {
				continue;
			}
			
			$subentities[] = $ent;
		}
		
		if ($checked_only) {
			$this->subentities['checked_only'] = $subentities;
		} else {
			$this->subentities['all'] = $subentities;
		}
		
		return $subentities;
	}

				
	/*
		The method is called for the main entity only.
		
		Field Sets are one level array. We do not dig up hierarchy of subentities, all of them
		reside in the first level.
	*/
	public function getFieldSets() {

		$sets = array();
	
		$sets[$this->getEntityCode()] = array(
			'name'   => $this->getEntityName(),
			'code'   => $this->getEntityCode(),
			'fields' => $this->getEntityFields()
		);

		$subentities = $this->getSubentities();
		foreach ($subentities as $se) {
			$sets[$se->getEntityCode()] = array(
				'name'   => $se->getEntityName(),
				'code'   => $se->getEntityCode(),
				'fields' => $se->getEntityFields()
			);
		}

		return $sets;
	}
	

	function countRows() {
		$rows = 1;
		
		foreach ($subobjects as $obj) {
			$rows = max($rows, $obj->countRows($this->entity_code));
		}
	}

	protected function prepareData(&$data) {
		return true;
	}

	/*
		we overwrite current() method of iterator to return complex data instead of just
		an identifier.
	*/
	public function current() {

		$data = array();

		$data[$this->entity_code] = array();
		
		$data[$this->entity_code][0] = parent::current();
		
		$this->prepareData($data[$this->entity_code][0]);
		
		$subentities = $this->getSubentities(true);
		
		foreach ($subentities as $se) {
			$se->setConditions($data[$this->entity_code][0]);
			foreach ($se as $se_key => $se_item) {
				$data = array_merge_recursive($data, $se_item);
			}
		}

		return $data;
	}

	
	public function isSubentityFor($code) {
	
		if (in_array($code, $this->subentity_for)) {
			return true;
		}
	
		return false;
	}

	// getEntityParamFields
	//
	public function getParamFields() {
		return array();
	}


	public function isMainEntity() {
		return $this->is_main_entity;
	}

	/* abstract?
	*/
	protected function defineQuery() {
		$this->query = '';
	}

	public function beforeProcessExport() {
		
		$this->params = $this->export_model->getExportParams();
		
		if (isset($this->params['entity_params'])) {
			$this->entity_params = $this->params['entity_params'];
			if ($this->isMainEntity()) {
				$this->setConditions($this->params['entity_params']);
			}
		}
		
		return true;
	}

	/*
		Extends array with custom fields from the entity table
	*/
	protected function addCustomEntityFields(&$fields, $default_fields = null, $entity_code = '') {

		if (empty($entity_code)) {
			$entity_code = $this->entity_code;
		}
	
		$qry = $this->db->query('SHOW FIELDS FROM ' . DB_PREFIX . $entity_code);
		if (empty($qry->rows)) {
			return false;
		}

		if (is_null($default_fields)) {
			$default_fields = $this->getDefaultTableFields();
		}
		
		foreach ($qry->rows as $row) {
			if (in_array($row['Field'], $default_fields)) {
				continue;
			}
			
			$field = array(
				'field' => $row['Field'],
				'name'  => $row['Field'],
				'descr' => 'Custom field. Type: ' . $row['Type']
			);
			
			$fields[$entity_code . '.' . $row['Field']] = $field;
		}
		
		return true;
	}

	protected function getDefaultTableFields() {
	
		$fields = $this->getDefaultEntityFields();
		$fields = array_keys($fields);
		
		return $fields;
	}
	
	protected function getDefaultEntityFields() {
		return array();
	}
	
	public function getEntityFields() {

		$fields = $this->getDefaultEntityFields();
		$this->addCustomEntityFields($fields);
		
		return $fields;	
	}
	
//
// these functions for custom_field data, they can be moved to a trait later
//	

	protected function getCustomFields($location) {
		$this->load->model('customer/custom_field');
		$fields = $this->model_customer_custom_field->getCustomFields();
		
		if (empty($fields)) {
			return false;
		}
		
		$_fields = array();
		foreach ($fields as $fld) {
			if ($fld['location'] == $location) {
				$_fields[] = $fld;
			}
		}
		
		return $_fields;
	}

	protected function getCustomFieldsAsColumns($location) {
	
		$fields = $this->getCustomFields($location);
		$return = array();

		if (empty($fields)) {
			return $return;
		}
		
		foreach ($fields as $fld) {
		
			$el = array(
				'field' => $fld['name'],
				'name'  => $fld['name'],
			);
			
			$return['cf_' . $fld['custom_field_id']] = $el;
		}

		return $return;
	}
	

	protected function getCustomFieldValue($custom_field_value_id) {
	
		$query = $this->db->query("SELECT cfvd.* FROM " . DB_PREFIX . "custom_field_value cfv 
			INNER JOIN " . DB_PREFIX . "custom_field_value_description cfvd 
				ON (cfv.custom_field_value_id = cfvd.custom_field_value_id) 
			WHERE cfv.custom_field_value_id = '" . (int)$custom_field_value_id . "' 
			AND cfvd.language_id = '" . (int)$this->config->get('config_language_id') . "'
		");

		if (empty($query->row)) {
			return false;
		}		

		return $query->row['name'];
	}

	
	protected function isFieldWithFixedValues($custom_field_id) {
		
		$custom_field_id = (int) $custom_field_id;
		$fixed_fields = array('select', 'radio', 'checkbox');
	
		$qry = $this->db->query("SELECT * FROM " . DB_PREFIX . "custom_field WHERE
			custom_field_id = '" . $custom_field_id . "'
		");
		
		if (empty($qry->row)) {
			return false;
		}
		
		if (in_array($qry->row['type'], $fixed_fields)) {
			return true;
		}
		
		return false;
	}
	
	
	// customer group
	//
	protected function getCustomerGroupName($customer_group_id) {
	
		$qry = $this->db->query("SELECT cgd.name FROM " . DB_PREFIX . "customer_group cg
			INNER JOIN " . DB_PREFIX . "customer_group_description cgd
				ON cg.customer_group_id = cgd.customer_group_id 
			WHERE 
				cg.customer_group_id = '" . (int)$customer_group_id . "'"
		);

		if (empty($qry->row)) {
			return false;
		}

		return $qry->row['name'];
	}
	
	// country
	//
	protected function getCountry($country_id) {
	
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "country WHERE country_id = '" . (int)$country_id . "'");

		return $query->row;
	}

	// zone
	//
	protected function getZone($zone_id) {
	
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone WHERE zone_id = '" . (int)$zone_id . "'");

		return $query->row;
	}

	// zone
	//
	protected function getLanguageName($id) {
	
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "language WHERE id = '" . (int)$id . "'");

		if (empty($this->query['name'])) {
			return '';
		}
		
		return $query->row['name'];
	}

	// get order status name
	//
	protected function getOrderStatusName($id) {
	
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_status WHERE order_status_id = '" . (int)$id . "'");

		if (empty($query->row['name'])) {
			return '';
		}
		
		return $query->row['name'];
	}
		
	public function currencyFormat($value, $code, $rate) {

		$value = $this->currency->format($value, $code, $rate);
	
		$value = preg_replace('/^[^\d|.,]*|[^\d|.,]*|&nbsp;$/','', $value);

		return $value;	
	}
}