<?php
/*
	$Project: CSV Uniexport $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 2.0.0.6 $ ($Revision: 158 $)
*/

namespace extension\ka_extensions\ka_uniexport;

class ModelExport extends \KaModel {

	// constants
	protected $export_version = '2.0.0.0';
	
	protected $sec_per_cycle    = 10;
	protected $enclosure        = '"';
	protected $escape           = "\x00";
	protected $use_escape = false;

	protected $filter = null;

	// session variables
	protected $stat;
	protected $params;

	// class variables
	public $entities;
	protected $main_entity;	
	protected $kaformat;
	protected $kalog;	
	protected $messages;

	protected $module_prefix = '';
	
	function onLoad() {

		parent::onLoad();
	
		$php_version = phpversion();
		if (version_compare($php_version, '5.5.4', '>=')) {
			$this->use_escape = true;
		}

 		$this->kalog = new \Log('ka_uniexport.log');
		$this->kaformat = new \KaFormat($this->config->get('config_language_id'));
		
		// specify stat session variable
		$this->stat   = &$this->getSession('stat');
		$this->params = &$this->getSession('params');

		//
 		$upd = $this->getConfig('ka_pe_update_interval');
 		if ($upd >= 5 && $upd <= 25) {
 			$this->sec_per_cycle = $upd;
 		}

		$this->file = new \KaFileUTF8();
	}

			
	public function isDBPrepared() {

		$res = $this->kadb->safeQuery("SHOW TABLES LIKE '" . DB_PREFIX . "ka_uniexport_profiles'");

		if (empty($res->rows)) {
			return false;
		}

		return true;
	}

	public function getExportStat() {
	 	return $this->stat;
	}

	public function addExportMessage($msg, $type = 'W') {

		$prefix = '';
		if ($type == 'W') {
			$prefix = 'WARNING';
		} else if ($type == 'E') {
			$prefix = 'ERROR';
		} elseif ($type == 'I') {
			$prefix = 'INFO';
		}

		if (!empty($this->messages) && count($this->messages) > 200) {
			$too_many = true;
			$msg = "too many messages...";
		} else {
			$msg = $prefix . ': ' . $msg;
		}

		$this->kalog->write("Message: " . $msg);

		$this->messages[] = $msg;
	}
	
	public function getExportMessages() {
		return $this->messages;
	}


	public function getSecPerCycle() {
		return $this->sec_per_cycle;
	}

	
 	protected function report($msg) {

 		if (defined('KA_DEBUG')) {
 			echo $msg;
 		}

		$this->kalog->write($msg);
 	}

		
	public function getRegisteredEntities() {
		
		if (empty($this->entities)) {
			$this->registerEntities();
		}

		return $this->entities;	
	}

	
	protected function getEntitiesLocation() {
		$dir = 'extension/ka_extensions/ka_uniexport/entities/';
		return $dir;
	}
	
	
	protected function registerEntities() {
		
		$dir = $this->getEntitiesLocation();
		
		$dir = DIR_APPLICATION . 'model/' . $dir;
		
		$files = glob($dir . '*.php');

		foreach ($files as $file) {
			$data = explode('/', dirname($file));
			$entity_code = basename($file, '.php');
			$models[$entity_code] = $entity_code;
		}
		
		if (empty($models)) {
			return false;
		}

		$entity_models = array();
		foreach ($models as $code => $model) {
			$obj = $this->loadEntityModel($model);
			if (!empty($obj)) {
				$entity_models[$code] = $obj;
			}
		}

		$this->entities = $entity_models;
	}

	
	public function loadEntityModel($model) {

		$dir = $this->getEntitiesLocation();
		$model = $dir . $model;
		
		$this->load->kamodel($model);

		$model_name = 'model_' . str_replace('/', '_', $model);
		
		
		if (!is_object($this->$model_name)) {
			return false;
		}
		
		if (!$this->isCompatible($this->$model_name)) {
			return false;
		}
		
		$this->$model_name->setExportModel($this);
		
		return $this->$model_name;
	}
	
	protected function isCompatible($model) {

		if (version_compare($this->export_version, $model->min_export_version, '<')
			|| version_compare($this->export_version, $model->max_export_version, '>'))
		{
			return false;
		}
		
		return true;
	}
	

	public function setMainEntity($ent) {

		$this->getRegisteredEntities();

		if (!is_object($this->entities[$ent])) {
			return false;
		}
		
		$this->main_entity = &$this->entities[$ent];
		
		return true;
	}

	/*
	
				'name' => array(
					'field' => 'name',
					'name'  => 'Option Name',
					'descr' => 'required'
				),

	*/
	public function getFieldSets() {

		$fieldSets = $this->main_entity->getFieldSets();

		return $fieldSets;
	}


	/*
		returns an array with column names
	*/
	protected function getHeader($columns) {
		
		$header = array();
		$sets = $this->getFieldSets();
		
		if (empty($sets)) {
			return $header;
		}
		
		foreach ($sets as $sk => $sv) {
		
			if (!empty($columns[$sk]) && is_array($columns[$sk])) {
				foreach ($columns[$sk] as $ck => $cv) {
					if (empty($cv['column'])) {
						$cv['column'] = $sv['name'] . ':' . $ck;
				 	}
				 	$header[] = htmlspecialchars_decode($cv['column']);
				}
			}
		}

		return $header;
	}
		
	
	protected function countRows($item, $columns) {

		if (empty($item) || empty($columns)) {
			$this->report(__METHOD__ . ": Wrong data in countRows");
			return false;
		}

		if ($this->params['data_in_one_cell']) {
			return 1;
		}
		
		$sets = array_keys($columns);

		$rows_required_num = 1;
		foreach ($sets as $sk => $sv) {
			if (!empty($item[$sv]) && is_array($item[$sv])) {
				$rows_required_num = max($rows_required_num, count($item[$sv]));
			}
		}

		return $rows_required_num;
	}
		
	/*
		column_sets - list of column sets
	*/
	protected function fillRow($item, $index) {
	
		$all_in_one = $this->params['data_in_one_cell'];
		
		$sep = ':::';
		$row = array();

		
		foreach ($this->params['columns'] as $set_code => $columns) {
		
			foreach ($columns as $col_code => $column) {
			
				if (empty($column['checked'])) {
					continue;
				}
				
				$val = '';
				if (!empty($column['is_key']) && $set_code == $this->params['main_entity_code']) {
				
					if (!empty($item[$set_code][0][$col_code])) {
						$val = $item[$set_code][0][$col_code];
					}
					
				} else { 
				
					$idx = $index;
					if (!empty($this->params['main_entity_to_each_row'])) {
						if ($set_code == $this->params['main_entity_code']) {
							$idx = 0;
						}
					}
				
					if (isset($item[$set_code][$idx])) {
					
						if (isset($item[$set_code][$idx][$col_code])) {

							if ($all_in_one && $idx == 0 && count($item[$set_code]) > 1) {

								foreach ($item[$set_code] as $ik => $iv) {
								
									$v = '';
									if (!empty($iv[$col_code])) {
									
										if (!empty($cv['type'])) {
											$v = $this->format($iv[$col_code], $cv['type']);
										} else {
											$v = $iv[$col_code];
										}
									}
									
									if (!empty($v)) {
										if (!empty($val)) {
											$val .= $sep;
										}
										$val .= $v;
									}
								}
								
							} else {
								$val = $item[$set_code][$idx][$col_code];
							}
						}					
					}
				}
				
				$row[] = $val;				
			}
		}

		
		return $row;
	}
	
	
	/*
		it is called once before starting the whole import process
	*/
	public function initExport($params) {
		
		if (empty($params['file'])) {
			$this->lastError = "File parameter is not defined.";
			return false;
		}
		
		// truncate the file
		//
		if (!($h = $this->file->fopen($params['file'], 'w'))) {
			$this->lastError = "File ($params[file]) cannot be opened.";
			return false;
		}
		$this->file->fclose();

		if (!$this->setMainEntity($params['main_entity_code'])) {
			die(__METHOD__ . ': main entity is not set');
		}
		
		$this->main_entity->setConditions($params['entity_params']);

		$sets = $this->getFieldSets();
		
		// remove sets if they are not required in the current export
		//
		foreach($sets as $sk => $sv) {
			$set_name = $sk;
		
			if (isset($params['columns'][$set_name])) {
				$is_checked = false;
				if (is_array($params['columns'][$set_name])) {
					foreach ($params['columns'][$set_name] as $csk => $csv) {
						if (!empty($csv['checked'])) {
							$is_checked = true;							
						} else {
							unset($params['columns'][$set_name][$csk]);
						}
					}
				}

				if (!$is_checked) {
					unset($params['columns'][$set_name]);
				}
			}
		}
		
		$this->params = $params;
		
		// adjust the language information
		//
		if (empty($this->params['language_id'])) {
			$this->params['language_id'] = $this->config->get('config_language_id');
		}
		$this->params['prefix_with_space'] = ($this->getConfig('ka_ue_prefix_with_space') == 'Y') ? true : false;
		$this->params['data_in_one_cell']  = ($this->getConfig('ka_ue_data_in_one_cell') == 'Y') ? true : false;

		$this->stat = array(
			'offset'           => 0,

			'started_at'       => time(),

			'lines_processed'    => 0,
			'products_processed' => 0,
			'products_total'     => 0,
			'file_size'          => 0,

			'errors'           => array(),
			'status'           => 'not_started',
		);

		
		//  .          
		//
		$this->stat['products_total'] = iterator_count($this->main_entity);

		$this->kalog->write("Export started. Parameters: " . var_export($this->stat, true));
		return true;
	}

	
	function beforeProcessExport() {
		$this->kaformat = new \KaFormat($this->params['language_id']);
		
		$max_execution_time = @ini_get('max_execution_time');
		if ($max_execution_time > 5 && $max_execution_time < $this->sec_per_cycle) {
			$this->sec_per_cycle = $max_execution_time - 3;
		}

		if (!$this->setMainEntity($this->params['main_entity_code'])) {
			die(__METHOD__ . ': main entity is not set');
		}
		
		$this->main_entity->setExportFields($this->params['columns']);
		
		$entities = $this->getRegisteredEntities();
		foreach ($entities as $entity) {
			$entity->beforeProcessExport();
		}
		
		return true;
	}
	
	/*
		called every import step update
	*/
	function processExport() {

		if ($this->stat['status'] == 'completed') {
			return false;
		}

		if (!$this->beforeProcessExport()) {
			return false;
		}
		
		$started_at = time();

		if (($handle = $this->file->fopen($this->params['file'], "a", array(), $this->params['charset'])) == FALSE) {
			$this->addExportMessage("Cannot open file: " . $this->params['file'], 'E');
			$this->stat['status']  = 'error';
			return;
		}
		
		if (empty($this->params['columns'])) {
			$this->addExportMessage("Columns are undefined", 'E');
			$this->stat['status']  = 'error';
			return;
		}

		// get store information
		//
		if ($this->params['store_id'] > 0) {
		
			$qry = $this->db->query("SELECT * FROM " . DB_PREFIX . "store WHERE 
				store_id = '" . (int)$this->params['store_id'] . "'"
			);
			if (empty($qry->row)) {
				$this->addExportMessage("Store not found", 'E');
				$this->stat['status']  = 'error';
				return;
			}
			$this->store_info = $qry->row;
			
		} else {
		
			// for products not belonging to any store we will use default store values
			// i.e. we use store_id = 0 instead of store_id = -1
			//
			$this->store_info = array(
				'store_id' => 0,
				'name'     => $this->config->get("config_name"),
				'url'      => HTTP_CATALOG
			);
		}

		$from = $this->stat['products_processed'];

		if ($from == 0) {
			$row = $this->getHeader($this->params['columns']);
			
			
			if ($this->use_escape) {
				$put_res = fputcsv($handle, $row, $this->params['delimiter'], $this->enclosure, $this->escape);
			} else {
				$put_res = fputcsv($handle, $row, $this->params['delimiter'], $this->enclosure);
			}
			
			if (!$put_res) {
				$this->addExportMessage("Cannot write header to '$this->params[file]'", 'E');
				return;
			}
		}

		$products_per_request = 50;
		$this->main_entity->setLimits($from, $products_per_request);

		$status = '';

		if (!$this->main_entity->isEmpty()) {

			foreach ($this->main_entity as $item) {

				$this->stat['products_processed']++;
			
				$rows_count = $this->countRows($item, $this->params['columns']);
				
				for ($i = 0; $i < $rows_count; $i++) {
					$row = $this->fillRow($item, $i);

					if (empty($row)) {
						$this->report("Empty product data. Productid=$pv[product_id]");
						continue;
					}

					if ($this->use_escape) {
						$put_res = fputcsv($handle, $row, $this->params['delimiter'], $this->enclosure, $this->escape);
					} else {
						$put_res = fputcsv($handle, $row, $this->params['delimiter'], $this->enclosure);
					}
					
					if (!$put_res) {
						$this->report("Nothing was written to $pv[product_id]");
						continue;
					}					

					$this->stat['lines_processed']++;	
				}
				
				if (time() - $started_at > $this->sec_per_cycle) {
					$status = 'in_progress';
					break;
				}
			}

			if ($this->main_entity->isEnd()) {
				$status = "completed";
			}
		} else {
			$status = 'completed';
		}
		
		$this->file->fclose();
		
		clearstatcache();
		$this->stat['file_size'] = filesize($this->params['file']);

	    if ($status == 'completed') {
    		$this->stat['status'] = 'completed';
    		$this->kalog->write("Export completed. Parameters: " . var_export($this->stat, true));

    		if (!empty($this->params['copy_path'])) {
    		
    			try {
    				$store_root_dir = dirname(DIR_APPLICATION);
	    			copy($this->params['file'], $store_root_dir . '/' . $this->kaformat->strip($this->params['copy_path'], array('/', '\\')));
	    			
	    		} catch (Exception $err) {
	    			$this->kalog->write("File not copied to:" . $this->params['copy_path']);
	    		}
    		}
    		
	    } else if ($status == 'error') {
			$this->stat['status'] = $status;

		} else {
			$this->stat['status'] = 'in_progress';
		}

    	return;
	}
	
	
	public function getCharsets() {
		$arr = array(
			'ISO-8859-1'   => 'ISO-8859-1 (Western Europe)',
			'ISO-8859-5'   => 'ISO-8859-5 (Cyrillc, DOS)',
			'KOI-8R'       => 'KOI-8R (Cyrillic, Unix)',
			'UTF-16LE'     => 'UNICODE (MS Excel text format)',
			'UTF-8'        => 'UTF-8',
			'windows-1250' => 'windows-1250 (Central European languages)',
			'windows-1251' => 'windows-1251 (Cyrillc)',
			'windows-1252' => 'windows-1252 (Western languages)',
			'windows-1253' => 'windows-1253 (Greek)',
			'windows-1254' => 'windows-1254 (Turkish)',
			'windows-1255' => 'windows-1255 (Hebrew)',
			'windows-1256' => 'windows-1256 (Arabic)',
			'windows-1257' => 'windows-1257 (Baltic languages)',
			'windows-1258' => 'windows-1258 (Vietnamese)',
			'big5'         => 'Chinese Traditional (Big5)',
			'CP932'        => 'CP932 (Japanese)',
		);

		return $arr;
	}
	
	
	public function tempname($dir, $prefix, $ext = 'tmp') {
		
		$try = 0;
		$res = false;
		while ($try++ < 100) {
			$name = $dir . $prefix . date("h_i_s") .  '_' . mt_rand(1, 999999) . '.' . $ext;

			if (!file_exists($name)) {
				if (file_put_contents($name, '') !== false) {
					$res = $name;
					break;
				}
			}
		}
		
		return $res;
	}


	public function getProfiles($entity_code = '') {
	
		$where = '';
		if (!empty($entity_code)) {
			$where = " WHERE entity_code = '" . $this->db->escape($entity_code) . "'";
		}
		$qry = $this->db->query("SELECT export_profile_id, name FROM " . DB_PREFIX . "ka_uniexport_profiles " . $where);

		$profiles = array();				
		if (!empty($qry->rows)) {
			foreach ($qry->rows as $row) {
				$profiles[$row['export_profile_id']] = $row['name'];
			}
		}
		
		return $profiles;
	}


	public function deleteProfile($profile_id) {

		$this->db->query("DELETE FROM " . DB_PREFIX. "ka_uniexport_profiles WHERE export_profile_id = '" . $this->db->escape($profile_id) . "'");
			
		return true;
	}
	
		
	/*
		returns:
			array - on success
			false - on error
	*/
	public function getProfileParams($profile_id) {
	
		$qry = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_uniexport_profiles WHERE export_profile_id = '" . $this->db->escape($profile_id) . "'");
		if (empty($qry->rows)) {
			return false;
		}

		if (!empty($qry->row['params'])) {
			$params = unserialize($qry->row['params']);
		} else {
			$params = array();
		}

		return $params;
	}
	
	
	/*
		returns:
			true  - on success
			false - on error
	*/
	public function setProfileParams($profile_id, $name, $params) {
	
		if (empty($profile_id)) {
		
			if (empty($params['main_entity_code'])) {
				$this->report(__METHOD__ . ": main entity code is not set");
				return false;
			}
		
			$this->db->query("INSERT INTO " . DB_PREFIX . "ka_uniexport_profiles
				SET 
					name = '" . $this->db->escape($name) . "',
					entity_code = '" . $this->db->escape($params['main_entity_code']) . "'
			");
			$profile_id = $this->db->getLastId();
		}
		
		$this->db->query("UPDATE " . DB_PREFIX . "ka_uniexport_profiles
			SET 
				name = '" . $this->db->escape($name) . "',
				params = '" . $this->db->escape(serialize($params)) . "'
			WHERE
				export_profile_id = '" . $this->db->escape($profile_id) . "'
		");

		return true;
	}
	
	public function getExportParams() {
		return $this->params;
	}


	public function format($val, $type = 'text', $format = '') {
	
		if ($type == 'date' || $type ='text') {
			if (!empty($this->params['prefix_with_space'])) {
				$val = ' ' . $val;
			}
		}
		
		return $val;
	}
	
	protected function getConfig($setting) {
		if (empty($this->module_prefix)) {
			$val = $this->config->get($setting);
		} else {
			$val = $this->config->get($this->module_prefix . '_' . $setting);
		}
		return $val;
	}
}